/*Author: 590684
 * Date: Jan 22, 2020
 * Description: Finding the distance of an object
 * than displaying it on a 4 digit, 7 segment display
 * and using a servo as a gauge
 */

//libraries 
#include <Servo.h>

//Pins for Distance Sensor
const int trigPin = A1;
const int echoPin = 3;

//Pins for 4 Digit 7 Seg Display
int digit1 = 9;
int digit2 = 10;
int digit3 = 11;
int num1, num2, num3;

int segA = A2;
int segB = 12;
int segC = 2;
int segD = 6;
int segE = A0;
int segF = 7;
int segG = 13;

//Variables for distance sensor and servo
long duration;
int distance;
int number;
int angle = 0;
Servo servo;

//Defining values for 7 seg display
#define DIGIT_ON  LOW
#define DIGIT_OFF  HIGH
#define DISPLAY_SPEED  3
#define SEGMENT_ON  HIGH
#define SEGMENT_OFF LOW


void setup() {
  //Pinmode for distance sensor
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  //Pinmode for for 7 seg display 
  pinMode(segA, OUTPUT);
  pinMode(segB, OUTPUT);
  pinMode(segC, OUTPUT);
  pinMode(segD, OUTPUT);
  pinMode(segE, OUTPUT);
  pinMode(segF, OUTPUT);
  pinMode(segG, OUTPUT);

  pinMode(digit1, OUTPUT);
  pinMode(digit2, OUTPUT);
  pinMode(digit3, OUTPUT);

  //Pinmode for LEDS and Servo
  pinMode (A3, OUTPUT);
  pinMode (8 , OUTPUT);
  servo.attach(5);

  Serial.begin(9600);
}

void loop() {
  //Sending PWM signal to distance sensor
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);

  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  //Recieving echo from distance sensor
  duration = pulseIn(echoPin, HIGH);

  //Calculating Distance
  distance = duration * 0.034 / 2;
  Serial.print("Distance: ");
  Serial.println(distance);

  //Splitting Distance into 3 digits
  num1 = (distance / 1) % 10;
  num2 = (distance / 10) % 10;
  num3 = (distance / 100) % 10;

  //Determing the angle for servo and slowly
  //moving to the left as distance gets smaller
  if (distance > 50){
    angle = 160;
    digitalWrite(A3, HIGH);
  }
  else if (40 < distance){
    angle = (120 + distance/1.5);
  }
  else if (30 < distance){
    angle = (90 + distance);
  }
  else if (15< distance){
    angle = (45 + distance*2);
  }
  //If distance is less than 15 than a red LED turns on
  else{
    angle = (25 + distance*2);
    digitalWrite(8, HIGH);
  }  
  servo.write(angle);
     
  //Displaying distance by constanly switching 
  //from one digit (display) to another
  for (int i =0; i < 61; i++) {
    digitalWrite(digit3, DIGIT_ON);
    convert (num1);
    delay(DISPLAY_SPEED);
    digitalWrite(digit3, DIGIT_OFF);

    digitalWrite(digit2, DIGIT_ON);
    Serial.println (num2);
    convert (num2);
    delay(DISPLAY_SPEED);
    digitalWrite(digit2, DIGIT_OFF);

    digitalWrite(digit1, DIGIT_ON);
    convert (num3);
    delay(DISPLAY_SPEED);
    digitalWrite(digit1, DIGIT_OFF);

  }

  //Turning off the LEDs
  digitalWrite(8, LOW);
  digitalWrite(A3, LOW);
}

//Takes the value of the digit and allow
//a single display flash a number based on input
void convert (int a) {
  if (a == 1) {
    Number1();
  }
  else if (a == 2) {
    Number2();
  }
  else if (a == 3) {
    Number3();
  }
  else if (a == 4) {
    Number4();
  }
  else if (a == 5) {
    Number5();
  }
  else if (a == 6) {
    Number6();
  }
  else if (a == 7) {
    Number7();
  }
  else if (a == 8) {
    Number8();
  }
  else if (a == 9) {
    Number9();
  }
  else if (a == 0) {
    Number0();
  }
  else {
    Number10();
  }
}

//Creates a number by turning on certain segments
//Does it from 0 to 9 and includes an all off 
void Number0() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_ON);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_OFF);
}


void Number1() {
  digitalWrite(segA, SEGMENT_OFF);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_OFF);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_OFF);
  digitalWrite(segG, SEGMENT_OFF);
}

void Number2() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_OFF);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_ON);
  digitalWrite(segF, SEGMENT_OFF);
  digitalWrite(segG, SEGMENT_ON);
}

void Number3() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_OFF);
  digitalWrite(segG, SEGMENT_ON);
}

void Number4() {
  digitalWrite(segA, SEGMENT_OFF);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_OFF);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_ON);
}

void Number5() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_OFF);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_ON);
}

void Number6() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_OFF);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_ON);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_ON);
}

void Number7() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_OFF);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_OFF);
  digitalWrite(segG, SEGMENT_OFF);
}

void Number8() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_ON);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_ON);
}

void Number9() {
  digitalWrite(segA, SEGMENT_ON);
  digitalWrite(segB, SEGMENT_ON);
  digitalWrite(segC, SEGMENT_ON);
  digitalWrite(segD, SEGMENT_ON);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_ON);
  digitalWrite(segG, SEGMENT_ON);
}

void Number10() {
  digitalWrite(segA, SEGMENT_OFF);
  digitalWrite(segB, SEGMENT_OFF);
  digitalWrite(segC, SEGMENT_OFF);
  digitalWrite(segD, SEGMENT_OFF);
  digitalWrite(segE, SEGMENT_OFF);
  digitalWrite(segF, SEGMENT_OFF);
  digitalWrite(segG, SEGMENT_OFF);
}
